package MT::Plugin::RebuildParentCategories;
use strict;
use MT;
use MT::Plugin;
use MT::Entry;
use MT::FileInfo;
use MT::WeblogPublisher;

our $VERSION = '0.2';

use base qw( MT::Plugin );

@MT::Plugin::RebuildParentCategories::ISA = qw( MT::Plugin );

my $plugin = new MT::Plugin::RebuildParentCategories( {
    name => 'RebuildParentCategories',
    id   => 'RebuildParentCategories',
    key  => 'rebuildparentcategories',
    description => 'Rebuild category archives for any parent categories for the categories when the entry saved.',
    author_name => 'Junnama Noda',
    author_link => 'http://junnama.alfasado.net/online/',
    version   => $VERSION,
} );

MT->add_plugin( $plugin );

sub init_registry {
    my $plugin = shift;
    $plugin->registry( {
        callbacks => {
            'cms_post_save.entry'
                => \&_post_save_cb,
            'cms_post_delete.entry'
                => \&_post_save_cb,
        },
   } );
}

# FIXME: Publish at Scheduled Task

sub _post_save_cb {
    my ( $cb, $app, $entry, $original ) = @_;
    my $rebuild;
    if ( $original ) {
        if ( ( $original->status == MT::Entry::RELEASE() ) &&
             ( $entry->status != MT::Entry::RELEASE() ) ) {
            $rebuild = 1;
        }
    }
    if ( $entry->status == MT::Entry::RELEASE() ) {
        $rebuild = 1;
    }
    if ( $rebuild ) {
        my $blog = $app->blog;
        my $pub = MT::WeblogPublisher->new;
        for my $cat ( @{ $entry->categories || [] } )  {
            next if (! $cat );
            for my $parent ( $cat->parent_categories ) {
                next if (! $parent );
                my @fis = MT::FileInfo->load( {
                                               blog_id => $blog->id,
                                               archive_type => 'Category',
                                               category_id => $parent->id,
                                             }, );
                if ( @fis ) {
                    for my $fi ( @fis ) {
                        my $res = $pub->rebuild_from_fileinfo( $fi );
                    }
                }
            }
        }
    }
}

1;